# RPM Packaging
## Packages
You will need the following packages for creating a RPM.
```bash
sudo dnf install fedora-packager fedora-review
```

## Enviroment
## User
Building and packaging should never be done as a user, instead add yourself to the `mock` group. The `mock` group allows you to use the `mock` package which:
> ...takes an SRPM and builds it in a chroot.

```bash
usermod -a -G mock YOUR_USERNAME_HERE
```

## Build directory
Now we need to setup a basic build enviroment for building rpms.
```bash
fedora-packager-setup
```

Get familar with your build enviroment:
```bash
tree ~/packaging-work/
```

## Building
### First-steps
Now we can begin building & packaging our package, create a directory in `~/packaging-work/` for the package `boilerplate`.

```bash
mkdir -p ~/packaging-work/boilerplate
```

Now copy the contents of `boilerplate` into the directory, either via `git`:

```bash
cd ~/packaging-work/boilerplate
git clone https://github.com/julianrichen/boilerplate-for-c-gtk3.git .
```

or `wget` the latest master (or stable branch):

```bash
cd ~/packaging-work/boilerplate
wget -qO- https://github.com/julianrichen/boilerplate-for-c-gtk3/archive/master.tar.gz | tar xvz -c ~/packaging-work/boilerplate
```

Or another method if you wish.

### Target
Now we use `fedpkg` to target a release version. Make sure you are in the correct working directory (`cd ~/packaging-work/boilerplate`).

```bash
fedpkg --release f<fedora_version> local
```

The package should now build and two rpms created, wooh!

### Lint
We should check that the rpm is valid.

```bash
fedpkg --release f<fedora_version> lint
```

If warnings/error appear, fix them now.

### Build
Now we are going to build the package with `mock`. Mock will download a minimumal Fedora instance for the package to be instlaled it. This will take a bit of time as a good amount of files need to be downloaded. The files will be caches so subsequent builds will be faster.

```bash
fedpkg --release f<fedora_version> mockbuild
```

### Review
In order for the package to be accepted we need to review the package with `fedora-review`.

```bash
cd ~/packaging-work/boilerplate
rm -rf *rpm results*

fedpkg --release f<fedora_version> srpm
fedora-review -n boilerplate
```

Check for issues, and fix any that exists. If none, send rpm to be reviewed.

## Fast package
A quick copy-paste block of commands to go from no package to complete rpm. You should really know what this stuff does but feel free to copy-paste away:
```bash
sudo dnf install fedora-packager fedora-review
usermod -a -G mock YOUR_USERNAME_HERE
fedora-packager-setup
mkdir -p ~/packaging-work/boilerplate
cd ~/packaging-work/boilerplate
git clone https://github.com/julianrichen/boilerplate-for-c-gtk3.git .
fedpkg --release f<fedora_version> srpm
fedora-review -n boilerplate
```

## Sources
+ https://fedoraproject.org/wiki/How_to_create_an_RPM_package
+ https://fedoraproject.org/wiki/Packaging
+ https://fedoraproject.org/wiki/Packaging:Guidlines
+ https://fedoraproject.org/wiki/Packaging:RPMMacros
+ http://rpm-guide.readthedocs.io/en/latest/rpm-guide.html
