#define G_LOG_DOMAIN "boilerplate-dark-theme"

#include "boilerplate-dark-theme.h"

#include "boilerplate-activatable.h"
#include "boilerplate-application.h"

#include <gtk/gtk.h>
#include <libpeas/peas.h>

typedef struct _BoilerplatePluginDarkThemePrivate
{
    BoilerplateApplication *application;
} BoilerplatePluginDarkThemePrivate;

struct _BoilerplatePluginDarkTheme
{
    PeasExtensionBase parent;
};

G_DEFINE_DYNAMIC_TYPE_EXTENDED (BoilerplatePluginDarkTheme,
                                boilerplate_dark_theme,
                                PEAS_TYPE_EXTENSION_BASE,
                                0,
                                G_IMPLEMENT_INTERFACE_DYNAMIC (BOILERPLATE_TYPE_ACTIVATABLE,
                                                               boilerplate_activatable_iface_init)
                                G_ADD_PRIVATE_DYNAMIC (BoilerplatePluginDarkTheme))

enum {
    PROP_0,
    PROP_APPLICATION,
    LAST_PROP
};

static void
boilerplate_dark_theme_activate (BoilerplateActivatable *activatable)
{
    g_object_set (gtk_settings_get_default (),
                  "gtk-application-prefer-dark-theme", TRUE,
                  NULL);
}

static void
boilerplate_dark_theme_deactivate (BoilerplateActivatable *activatable)
{
    g_object_set (gtk_settings_get_default (),
                  "gtk-application-prefer-dark-theme", FALSE,
                  NULL);
}

static void
boilerplate_dark_theme_update_state (BoilerplateActivatable *activatable)
{
}

static void
boilerplate_activatable_iface_init (BoilerplateActivatableInterface *iface)
{
    iface->activate = boilerplate_dark_theme_activate;
    iface->deactivate = boilerplate_dark_theme_deactivate;
    iface->update_state = boilerplate_dark_theme_update_state;
}

static void
boilerplate_dark_theme_get_property (GObject    *object,
                                            guint       prop_id,
                                            GValue     *value,
                                            GParamSpec *pspec)
{
    BoilerplatePluginDarkTheme *self = BOILERPLATE_DARK_THEME (object);
    BoilerplatePluginDarkThemePrivate *priv = boilerplate_dark_theme_get_instance_private (self);

    switch (prop_id)
    {
        case PROP_APPLICATION:
            g_value_set_object (value, priv->application);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    }
}

static void
boilerplate_dark_theme_set_property (GObject      *object,
                                     guint         prop_id,
                                     const GValue *value,
                                     GParamSpec   *pspec)
{
    BoilerplatePluginDarkTheme *self = BOILERPLATE_DARK_THEME (object);
    BoilerplatePluginDarkThemePrivate *priv = boilerplate_dark_theme_get_instance_private (self);

    switch (prop_id)
    {
        case PROP_APPLICATION:
            priv->application = BOILERPLATE_APPLICATION (g_value_dup_object (value));
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    }
}

static void
boilerplate_dark_theme_class_finalize (BoilerplatePluginDarkThemeClass *klass)
{
}

static void
boilerplate_dark_theme_class_init (BoilerplatePluginDarkThemeClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->get_property = boilerplate_dark_theme_get_property;
    object_class->set_property = boilerplate_dark_theme_set_property;

    g_object_class_override_property (object_class,
                                      PROP_APPLICATION,
                                      "application");
}

static void
boilerplate_dark_theme_init (BoilerplatePluginDarkTheme *self)
{
}

G_MODULE_EXPORT void
peas_register_types (PeasObjectModule *module)
{
    boilerplate_dark_theme_register_type (G_TYPE_MODULE (module));

    peas_object_module_register_extension_type (module,
                                                BOILERPLATE_TYPE_ACTIVATABLE,
                                                BOILERPLATE_TYPE_DARK_THEME);
}
